# This script conducts OLS regression analysis using two measures of 
# delinquency in the Survey of Consumer Finances as the outcome. Explanatory
# variables include demographics for race/ethnicity, education, and age. Other
# categories included balance sheet characteristics, family structure, and
# idiosyncratic "luck". Two structural models are used, the first posits
# that choice sets are equal across individuals and the coefficients on racial
# binaries is accounted for by observable explantory variables. The second model 
# allows choice sets to be defined by demographics. Explanatory variables are 
# de-meaned with means constructed by demographic groupings. The regression
# results are featured in "College is Not Enough: Higher Education Does Not
# Eliminate Racial and Ethnic Wealth Gaps", by William R. Emmons and Lowell R. 
# Ricketts.
#
# This script was written by Lowell Ricketts, the Senior Analyst for the
# Center for Household Financial Stability at the Federal Reserve Bank of St.
# Louis. The author would like to thank Anthony Damico (www.asdfree.com) for
# foundational scripts and guidance for working with SCF data in R.

source.dir <- "Enter source directory here"
out.dir <- "Enter output directory here"

# load two svyttest functions (one to conduct a df-adjusted t-test and one to 
# conduct a multiply-imputed t-test)
source(paste0(source.dir, "/asdfree R scripts/scf.survey.R"))

# Load ihs.loglik function which calculates the value of the concentrated 
# log-likelihood function. This will be used estimate and confirm the optimal theta.
source(paste0(source.dir, "/ihs.loglik.R"))

library(mitools)               # allows analysis of multiply-imputed survey data
library(survey)          # load survey package (analyzes complex design surveys)
library(dplyr)                        # Quintessential data manipulation package
library(xlsx)                # Allows export of regression results to excel file
library(Hmisc)

# Omit years 1989 and 1992 because usual income question wasn't asked, therefore
# you cannot construct income shock variable during those years.
years.to.loop <- c(1995, 1998, 2001, 2004, 2007, 2010, 2013)

# Set smoothing parameter for inverse hyperbolic sine transformation of net worth
# See optimal theta generated later in script.
theta = 0.0001

results <- NULL
full_imp1 <- NULL
full_imp2 <- NULL
full_imp3 <- NULL
full_imp4 <- NULL
full_imp5 <- NULL
full_rw <- NULL

# CPI-U Annual Series sourced from Haver Analytics, vintage as of 6/27/16
cpi <- read.csv(paste0(source.dir, "/cpi_u_annual.csv"))

bcohort.loop <- function(x) {
        x <- mutate(x, birthyear = year - age)
        for (i in seq(1898, 2013, by = 5)) {
                temp <- 
                        ifelse(x$birthyear >= as.integer(i) & 
                                       x$birthyear <= as.integer(i) + 4, 1, 0)
                x[as.character(paste0("b_", as.integer(i) + 2))] = temp
        }
        x
}

deflate.gifts <- function(x) {
        # Correct small number of negative responses in x5804 and x5809
        x <- mutate(x,
                    x5804 = ifelse(x5804 < 0, 0, x5804),
                    x5809 = ifelse(x5809 < 0, 0, x5809),
                    x5814 = ifelse(x5814 < 0, 0, x5814))
        gift1 <- select(x, c(y1, yy1, year, x5804, x5805)) %>%
                left_join(cpi, by = c("x5805" = "cpi_year")) %>%
                mutate(gift1 = x5804/deflator) %>%
                select(y1, yy1, year, gift1)
        gift2 <- select(x, c(y1, yy1, year, x5809, x5810)) %>%
                left_join(cpi, by = c("x5810" = "cpi_year")) %>%
                mutate(gift2 = x5809/deflator) %>%
                select(gift2)
        gift3 <- select(x, c(y1, yy1, year, x5814, x5815)) %>%
                left_join(cpi, by = c("x5815" = "cpi_year")) %>%
                mutate(gift3 = x5814/deflator) %>%
                select(gift3)
        
        adj_gift <- cbind(gift1, gift2, gift3)
        
        adj_gift <- mutate(adj_gift, 
                           gift1 = ifelse(is.na(gift1), 0, gift1),
                           gift2 = ifelse(is.na(gift2), 0, gift2),
                           gift3 = ifelse(is.na(gift3), 0, gift3))
        
        adj_gift$real_gift = rowSums(adj_gift[4:6])
        
        adj_gift <- select(adj_gift, c(y1, yy1, year, real_gift))
        x <- left_join(x, adj_gift, by = c("y1", "yy1", "year"))
}

create.variables <- function(x) {
        x <- mutate(x,
                    usual_inc = ifelse(year %in% c(1989, 1992), pmax(1, income), 
                                       ifelse(x7650 == 3, pmax(1, income), 
                                              pmax(1, x7362))),
                    # Remittance to family indicator
                    fam_supp = ifelse(x5734 > 0, 1, 0),
                    # Share of income contributed to family as gift/remittance
                    fam_supp_share = ifelse(usual_inc > 1, 100 * x5734/usual_inc, 0),
                    ismarried = ifelse(married == 1, 1, 0),
                    hh_size = x101,
                    ## Dependent variables
                    adj_nw = networth / sqrt(hh_size),
                    adj_inc = income / sqrt(hh_size),
                    # IHS transformed net worth
                    ihs_nw = log(theta * networth + 
                                          sqrt((theta ^ 2) * (networth ^ 2) + 1))/theta,
                    ihs_adj_nw = log(theta * adj_nw +
                                             sqrt((theta ^ 2) * (networth ^ 2) + 1))/theta,
                    ## Balance sheet variables
                    # Home ownership indicator variable
                    home_owner = ifelse(houses > pmin(((1/52) * usual_inc), 1000), 1, 0),
                    # Share of assets invested in residential real estate
                    home_share = ifelse(asset == 0, 0, 100 * houses/asset),
                    # Durable goods ownership indicator
                    dg_owner = ifelse(rowSums(x[,c(grep("vehic",names(x)), 
                                                   grep("othnfin",names(x)))]) > 
                                              pmin(((1/52) * usual_inc), 1000), 1, 0),
                    # Share of assets invested in durable goods
                    dg_share = ifelse(asset == 0, 0, 
                                      100 * rowSums(x[,c(grep("vehic", names(x)), 
                                                         grep("othnfin", names(x)))]
                                                    )/asset),
                    # Net equity in residential real estate other than primary res.
                    net_oresre = oresre - resdbt,
                    # Financial assets aside from safe and liquid assets
                    net_fin = fin - rowSums(x[,c(grep("liq", names(x)), 
                                                 grep("cds", names(x)),
                                                 grep("savbnd", names(x)))]),
                    # Financial or business asset ownership indicator
                    finbus_owner = ifelse(
                            rowSums(x[,c(grep("net_oresre", names(x)), 
                                         grep("nnresre", names(x)),
                                         grep("bus", names(x)),
                                         grep("net_fin", names(x)))]) > 
                                    pmin(((1/52) * usual_inc), 1000), 1, 0),
                    # Share of assets invested in financial or business assets
                    finbus_share = ifelse(asset == 0, 0, 
                                          100 * rowSums(x[,c(grep("net_oresre", names(x)), 
                                                             grep("nnresre", names(x)),
                                                             grep("bus", names(x)),
                                                             grep("net_fin", names(x)))]
                                                        )/asset),
                    # Safe and liquid asset ownership indicator
                    sla_owner = ifelse(rowSums(x[,c(grep("liq", names(x)),
                                                    grep("cds", names(x)),
                                                    grep("savbnd", names(x)))]) > 
                                               pmin(((1/52) * usual_inc), 1000), 1, 0),
                    # Share of assets invested in safe and liquid assets
                    sla_share = ifelse(asset == 0, 0, 
                                       100 * rowSums(x[,c(grep("liq", names(x)),
                                                          grep("cds", names(x)),
                                                          grep("savbnd", names(x)))]
                                                     )/asset),
                    # Non-mortgage debt ownership indicator
                    nmd_owner = ifelse(rowSums(x[,c(grep("install", names(x)),
                                                    grep("othloc", names(x)),
                                                    grep("ccbal", names(x)), 
                                                    grep("odebt", names(x)))]) > 
                                               pmin(((1/52) * usual_inc), 1000), 1, 0),
                    # Share of non-mortgage debt to assets
                    nmd_share = ifelse(rowSums(x[,c(grep("install", names(x)),
                                                    grep("othloc", names(x)),
                                                    grep("ccbal", names(x)), 
                                                    grep("odebt", names(x)))]) > 0 
                                       & asset > 0,
                                       pmin(100 * rowSums(x[,c(grep("install", names(x)),
                                                          grep("othloc", names(x)),
                                                          grep("ccbal", names(x)), 
                                                          grep("odebt", names(x)))]
                                                     )/asset, 100),
                                       ifelse(rowSums(x[,c(grep("install", names(x)),
                                                           grep("othloc", names(x)),
                                                           grep("ccbal", names(x)), 
                                                           grep("odebt", names(x)))]) > 0 
                                              & asset == 0, 100, 0)),
                    # Mortgage debt ownership indicator
                    md_owner = ifelse(mrthel > pmin(((1/52) * usual_inc), 1000), 1, 0),
                    # Share of mortgage debt to assets
                    md_share = ifelse(mrthel > 0 & asset > 0, 
                                      pmin(100 * mrthel/asset, 100), 
                                      ifelse(mrthel > 0 & asset == 0, 100, 0)),
                    # Modified financial scorecard
                    fin_score = 1 * (x3018 == 1 | x3020 == 1) + 
                            1 - 1 * (x3004 == 5 | (!is.na(x7507) & x7507 == 3)) +
                            1 - 1 * ((!is.na(x7575) & x7575 > 0) | 
                                             x407 %in% c(1, 3) | x409 == 1) +
                            1 * (rowSums(x[,c(grep("liq", names(x)),
                                              grep("cds", names(x)),
                                              grep("savbnd", names(x)))]
                                         )/pmax(1, asset) > .1) +
                            1 * (pir40 == 0),
                    # Feels financially lucky indicator
                    lucky = ifelse(x6788 %in% c(1, 2), 1, 0),
                    ## Educational attainment variables
                    # Educational achievement class: 4 = Graduate degree, 3 = 2-4
                    # year college graduate, 2 = HS Diploma/ GED, 1 = HS dropout
                    edcl = ifelse(x5904 == 1 & (x5905 %in% c(3,4)), 4,
                                  ifelse(x5904 == 1 & (x5905 %in% c(1, 2, 11)), 3,
                                         ifelse((x5901 > 12 & x5904 == 5) | 
                                                        (x5902 %in% c(1,2)), 2, 
                                                ifelse(x5902 == 5, 1, NA)))),
                    # YY1 4556 in 2013 has a miscoded educational attainment
                    edcl = ifelse(x5901 > 12 & x5904 == 0, 2, edcl),
                    # Years of schooling completed
                    # NOTE: Assumes max number of years of undergraduate education is
                    # four years. Also codes graduate school as one additional year
                    # of schooling after undergraduate
                    school_yrs = ifelse(x5901 > 0, x5901, 0),
                    # Education binaries for regression
                    no_hs = ifelse(edcl == 1, 1, 0),
                    hs = ifelse(edcl == 2, 1, 0),
                    bs_assoc = ifelse(edcl == 3, 1, 0),
                    grad = ifelse(edcl == 4, 1, 0),
                    grad.4yr = ifelse(x5904 == 1 & (x5905 %in% c(2, 3, 4)), 1, 0),
                    ## Race and Ethnicity Variables
                    # Recode race variable 
                    race = ifelse(race == 1, 1,
                                  ifelse(race == 2, 2,
                                         ifelse(race == 3, 3, 4))),
                    # Race binaries for regression
                    iswhite = ifelse(race == 1, 1, 0),
                    isblack = ifelse(race == 2, 1, 0),
                    ishispanic = ifelse(race == 3, 1, 0),
                    isother = ifelse(race == 4, 1, 0),
                    ## Age Variables
                    age_young = ifelse(age <= 39, 1, 0),
                    age_middle = ifelse(age >= 40 & age <= 61, 1, 0),
                    age_old = ifelse(age >= 62, 1, 0),
                    agecl = ifelse(age <= 39, 1, 
                                   ifelse((age >= 40 & age <= 61), 2,
                                          ifelse(age >= 62, 3, NA))),
                    age2 = age ^ 2,
                    age3 = age ^ 3,
                    ## Luck Variables
                    # Income shock indicators
                    inc_high = ifelse(x7650 == 1, 1, 0),
                    inc_low = ifelse(x7650 == 2, 1, 0),
                    # General health categorized as excellent or good
                    healthy = ifelse(x6030 %in% c(1,2), 1, 0),
                    # Received an inheritance or gift indicator variable
                    gift = ifelse(real_gift > pmin(((1/52) * usual_inc), 1000), 1, 0),
                    # Share of gifts and inheritances as share of assets
                    gift_share = ifelse(asset == 0, 0, 100 * real_gift/asset),
                    ## Survey Year Indicator Variables
                    y1989 = ifelse(year == 1989, 1, 0), 
                    y1992 = ifelse(year == 1992, 1, 0),
                    y1995 = ifelse(year == 1995, 1, 0),
                    y1998 = ifelse(year == 1998, 1, 0),
                    y2001 = ifelse(year == 2001, 1, 0),
                    y2004 = ifelse(year == 2004, 1, 0),
                    y2007 = ifelse(year == 2007, 1, 0),
                    y2010 = ifelse(year == 2010, 1, 0),
                    y2013 = ifelse(year == 2013, 1, 0),
                    ## Year Groupings (For Normalization in Structural Specs.)
                    yr_group = ifelse(year %in% c(1995, 1998, 2001), 1, 
                                      ifelse(year %in% c(2004, 2007), 2, 3)),
                    ## Race & Education Interaction Variables
                    nohs.black = no_hs * isblack,
                    nohs.hispanic = no_hs * ishispanic,
                    nohs.other = no_hs * isother,
                    hs.black = hs * isblack,
                    hs.hispanic = hs * ishispanic,
                    hs.other = hs * isother,
                    grad.black = grad * isblack,
                    grad.hispanic = grad * ishispanic,
                    grad.other = grad * isother,
                    ## Data Cleaning - bound shares between 0 and 100
                    home_share = ifelse(home_share < 0, 0, home_share),
                    home_share = ifelse(home_share > 100, 100, home_share),
                    dg_share = ifelse(dg_share < 0, 0, dg_share),
                    dg_share = ifelse(dg_share > 100, 100, dg_share), 
                    finbus_share = ifelse(finbus_share < 0, 0, finbus_share),
                    finbus_share = ifelse(finbus_share > 100, 100, finbus_share), 
                    sla_share = ifelse(sla_share < 0, 0, sla_share),
                    sla_share = ifelse(sla_share > 100, 100, sla_share),
                    nmd_share = ifelse(nmd_share < 0, 0, nmd_share),
                    md_share = ifelse(md_share < 0, 0, md_share),
                    fam_supp_share = ifelse(fam_supp_share > 100, 100, fam_supp_share),
                    # Apply natural log to usual income after it has been used
                    # to create other variables
                    usual_inc = log(usual_inc))
}

for (i in seq( length( years.to.loop ) ) ) {
        load(paste0(source.dir, "/scf", years.to.loop[i], ".rda"))
        
        # define which variables from the five imputed iterations to keep
        vars.to.keep <- c( 'y1' , 'yy1' , 'year', 'wgt' , 'one' , 'five' ,
                           'x101', 'x407', 'x409', 'x3004', 'x3015', 'x3016',
                           'x3017', 'x3018', 'x3019', 'x3020', 'x5734', 'x5801',
                           'x5804', 'x5805', 'x5809', 'x5810', 'x5814', 'x5815', 
                           'x5901', 'x5902', 'x5904', 'x5905', 'x6030', 'x6788',
                           'x7362', 'x7507', 'x7508' ,'x7575', 'x7650', 
                           'networth', 'income', 'houses', 'asset', 'vehic',
                           'othnfin', 'oresre', 'resdbt', 'liq', 'cds', 'savbnd',
                           'fin', 'nnresre', 'bus', 'install', 'othloc', 'ccbal',
                           'odebt', 'mrthel', 'race', 'age', 'married', 'kids', 
                           'pir40')
        
        # Add SCF year as variable
        imp1 <- mutate(imp1, year = years.to.loop[i])
        imp2 <- mutate(imp2, year = years.to.loop[i])
        imp3 <- mutate(imp3, year = years.to.loop[i])
        imp4 <- mutate(imp4, year = years.to.loop[i])
        imp5 <- mutate(imp5, year = years.to.loop[i])
        #rw <- mutate(rw, year = years.to.loop[i])
        
        # restrict each `imp#` data frame to only those variables
        imp1 <- imp1[ , vars.to.keep ]
        imp2 <- imp2[ , vars.to.keep ]
        imp3 <- imp3[ , vars.to.keep ]
        imp4 <- imp4[ , vars.to.keep ]
        imp5 <- imp5[ , vars.to.keep ]
        
        # clear up RAM
        gc()
        
        # Adjust total amount of gifts and inheritances for inflation, put in 
        # terms of 2013 dollars.
        imp1 <- deflate.gifts(imp1)
        imp2 <- deflate.gifts(imp2)
        imp3 <- deflate.gifts(imp3)
        imp4 <- deflate.gifts(imp4)
        imp5 <- deflate.gifts(imp5)
        
        # Create binary indicator variables for birth cohort
        imp1 <- bcohort.loop(imp1)
        imp2 <- bcohort.loop(imp2)
        imp3 <- bcohort.loop(imp3)
        imp4 <- bcohort.loop(imp4)
        imp5 <- bcohort.loop(imp5)
        
        # Create all other variables for analysis
        imp1 <- create.variables(imp1)
        imp2 <- create.variables(imp2)
        imp3 <- create.variables(imp3)
        imp4 <- create.variables(imp4)
        imp5 <- create.variables(imp5)
        
        # Append new year to combined data set
        full_imp1 <- rbind(full_imp1, imp1)
        full_imp2 <- rbind(full_imp2, imp2)
        full_imp3 <- rbind(full_imp3, imp3)
        full_imp4 <- rbind(full_imp4, imp4)
        full_imp5 <- rbind(full_imp5, imp5)
        full_rw <- rbind(full_rw, rw)
        
}

rm(imp1, imp2, imp3, imp4, imp5, rw)

# Estimate optimal theta using full PR model, use it for ALL models.
theta.hat <- optimize(ihs.loglik, lower = .000001, upper = .2, maximum = TRUE, 
         depvar = full_imp1$networth,
         indvars = full_imp1[ ,c('isblack','ishispanic', 'isother', 'age',
                                 'age2', 'age3', 'y1998', 'y2001', 'y2004',
                                 'y2007', 'y2010', 'y2013', 'b_1900', 'b_1905',
                                 'b_1910', 'b_1915', 'b_1920', 'b_1925', 'b_1930',
                                 'b_1935', 'b_1945', 'b_1950', 'b_1955', 'b_1960',
                                 'b_1965', 'b_1970', 'b_1975', 'b_1980', 'b_1985',
                                 'b_1990', 'no_hs', 'hs', 'grad', 'ismarried',
                                 'kids', 'fam_supp', 'fam_supp_share', 'dg_owner',
                                 'dg_share', 'home_owner', 'home_share', 'sla_owner',
                                 'sla_share', 'finbus_owner', 'finbus_share',
                                 'nmd_owner', 'nmd_share', 'md_owner', 'md_share',
                                 'fin_score', 'inc_high', 'inc_low', 'gift',
                                 'healthy', 'nohs.black', 'nohs.hispanic',
                                 'nohs.other', 'hs.black', 'hs.hispanic', 
                                 'hs.other', 'grad.black', 'grad.hispanic',
                                 'grad.other')],
         weight = full_imp1$wgt)

paste0("Optimal Theta: ", round(theta.hat$maximum, 4))
paste0("Theta Used: ", theta)

# For basic coefficients and NOT standard errors, just ignore replicate weights. 
# Replicate design requires tremendous computational resources.
scf.design <-
        svydesign(
                ids = ~0,
                # use the main weight within each of the imp# objects
                weights = ~wgt,
                # read the data directly from the five implicates
                data = imputationList(list( full_imp1, full_imp2, full_imp3, 
                                             full_imp4, full_imp5))
        )

# Calculate means for variables as defined by peer groups (race/ethnicity, 
# age, and 3-survey group)
mu.school_yrs <- scf.MIcombine(with(scf.design,
                              svyby(~school_yrs, ~race + agecl + yr_group,
                                     svymean, na.rm = TRUE)))

mu.fam_supp <- scf.MIcombine(with(scf.design,
                                    svyby(~fam_supp, ~race + agecl + yr_group,
                                          svymean, na.rm = TRUE)))

mu.fam_supp_share <- scf.MIcombine(with(scf.design,
                                  svyby(~fam_supp_share, ~race + agecl + yr_group,
                                        svymean, na.rm = TRUE)))

mu.healthy <- scf.MIcombine(with(scf.design,
                                        svyby(~healthy, ~race + agecl + yr_group,
                                              svymean, na.rm = TRUE)))

mu.ismarried <- scf.MIcombine(with(scf.design,
                                        svyby(~ismarried, ~race + agecl + yr_group,
                                              svymean, na.rm = TRUE)))

mu.kids <- scf.MIcombine(with(scf.design,
                                        svyby(~kids, ~race + agecl + yr_group,
                                              svymean, na.rm = TRUE)))

mu.home_owner <- scf.MIcombine(with(scf.design,
                                        svyby(~home_owner, ~race + agecl + yr_group,
                                              svymean, na.rm = TRUE)))

mu.home_share <- scf.MIcombine(with(scf.design,
                                        svyby(~home_share, ~race + agecl + yr_group,
                                              svymean, na.rm = TRUE)))
mu.dg_owner <- scf.MIcombine(with(scf.design,
                                        svyby(~dg_owner, ~race + agecl + yr_group,
                                              svymean, na.rm = TRUE)))

mu.dg_share <- scf.MIcombine(with(scf.design,
                                        svyby(~dg_share, ~race + agecl + yr_group,
                                              svymean, na.rm = TRUE)))

mu.sla_owner <- scf.MIcombine(with(scf.design,
                                  svyby(~sla_owner, ~race + agecl + yr_group,
                                        svymean, na.rm = TRUE)))

mu.sla_share <- scf.MIcombine(with(scf.design,
                                  svyby(~sla_share, ~race + agecl + yr_group,
                                        svymean, na.rm = TRUE)))

mu.finbus_owner <- scf.MIcombine(with(scf.design,
                                   svyby(~finbus_owner, ~race + agecl + yr_group,
                                         svymean, na.rm = TRUE)))

mu.finbus_share <- scf.MIcombine(with(scf.design,
                                   svyby(~finbus_share, ~race + agecl + yr_group,
                                         svymean, na.rm = TRUE)))

mu.nmd_owner <- scf.MIcombine(with(scf.design,
                                      svyby(~nmd_owner, ~race + agecl + yr_group,
                                            svymean, na.rm = TRUE)))

mu.nmd_share <- scf.MIcombine(with(scf.design,
                                      svyby(~nmd_share, ~race + agecl + yr_group,
                                            svymean, na.rm = TRUE)))

mu.md_owner <- scf.MIcombine(with(scf.design,
                                   svyby(~md_owner, ~race + agecl + yr_group,
                                         svymean, na.rm = TRUE)))

mu.md_share <- scf.MIcombine(with(scf.design,
                                   svyby(~md_share, ~race + agecl + yr_group,
                                         svymean, na.rm = TRUE)))

mu.fin_score <- scf.MIcombine(with(scf.design,
                                  svyby(~fin_score, ~race + agecl + yr_group,
                                        svymean, na.rm = TRUE)))

mu.gift <- scf.MIcombine(with(scf.design,
                                   svyby(~gift, ~race + agecl + yr_group,
                                         svymean, na.rm = TRUE)))

mu.real_gift <- scf.MIcombine(with(scf.design,
                                   svyby(~real_gift, ~race + agecl + yr_group,
                                         svymean, na.rm = TRUE)))

mu.usual_inc <- scf.MIcombine(with(scf.design,
                                   svyby(~usual_inc, ~race + agecl + yr_group,
                                         svymean, na.rm = TRUE)))

mu.inc_low <- scf.MIcombine(with(scf.design, 
                                 svyby(~inc_low, ~race + agecl + yr_group, 
                                       svymean, na.rm = TRUE)))

mu.inc_high <- scf.MIcombine(with(scf.design, 
                                  svyby(~inc_high, ~race + agecl + yr_group, 
                                        svymean, na.rm = TRUE)))


ids <- attributes(mu.fam_supp$coefficients)$names
ids <- unlist(strsplit(ids, ".", fixed = TRUE))
race <- as.integer(ids[seq(1, 106, by = 3)])
agecl <- as.integer(ids[seq(2, 107, by = 3)])
yr_group <- as.integer(ids[seq(3, 108, by = 3)])

norms <- as.data.frame(
                cbind(race, agecl, yr_group,
                      as.vector(mu.fam_supp$coefficients),
                      as.vector(mu.fam_supp_share$coefficients),
                      as.vector(mu.dg_owner$coefficients),
                      as.vector(mu.dg_share$coefficients),
                      as.vector(mu.fin_score$coefficients),
                      as.vector(mu.finbus_owner$coefficients),
                      as.vector(mu.finbus_share$coefficients),
                      as.vector(mu.healthy$coefficients),
                      as.vector(mu.home_owner$coefficients),
                      as.vector(mu.home_share$coefficients),
                      as.vector(mu.ismarried$coefficients),
                      as.vector(mu.kids$coefficients),
                      as.vector(mu.md_owner$coefficients),
                      as.vector(mu.md_share$coefficients),
                      as.vector(mu.nmd_owner$coefficients),
                      as.vector(mu.nmd_share$coefficients),
                      as.vector(mu.gift$coefficients),
                      as.vector(mu.real_gift$coefficients),
                      as.vector(mu.school_yrs$coefficients),
                      as.vector(mu.sla_owner$coefficients),
                      as.vector(mu.sla_share$coefficients),
                      as.vector(mu.usual_inc$coefficients),
                      as.vector(mu.inc_low$coefficients),
                      as.vector(mu.inc_high$coefficients)))

names(norms) <- c("race", "agecl", "yr_group","mu.fam_supp", "mu.fam_supp_share",
                 "mu.dg_owner", "mu.dg_share", "mu.fin_score", "mu.finbus_owner",
                 "mu.finbus_share", "mu.healthy", "mu.home_owner", 
                 "mu.home_share", "mu.ismarried", "mu.kids", "mu.md_owner", 
                 "mu.md_share", "mu.nmd_owner", "mu.nmd_share", "mu.gift", 
                 "mu.real_gift", "mu.school_yrs", "mu.sla_owner", "mu.sla_share",
                 "mu.usual_inc", "mu.inc_low", "mu.inc_high")

normalize <- function(a) {
        a <- left_join(x = a, y = norms, by = c("race", "agecl", "yr_group")) %>%
                mutate(norm.fam_supp = fam_supp - mu.fam_supp,
                       norm.fam_supp_share = fam_supp_share - mu.fam_supp_share,
                       norm.dg_owner = dg_owner - mu.dg_owner,
                       norm.dg_share = dg_share - mu.dg_share,
                       norm.fin_score = fin_score - mu.fin_score,
                       norm.finbus_owner = finbus_owner - mu.finbus_owner,
                       norm.finbus_share = finbus_share - mu.finbus_share,
                       norm.healthy = healthy - mu.healthy,
                       norm.home_owner = home_owner - mu.home_owner,
                       norm.home_share = home_share - mu.home_share,
                       norm.ismarried = ismarried - mu.ismarried,
                       norm.kids = kids - mu.kids,
                       norm.md_owner = md_owner - mu.md_owner,
                       norm.md_share = md_share - mu.md_share,
                       norm.nmd_owner = nmd_owner - mu.nmd_owner,
                       norm.nmd_share = nmd_share - mu.nmd_share,
                       norm.gift = gift - mu.gift,
                       norm.real_gift = real_gift - mu.real_gift,
                       norm.school_yrs = school_yrs - mu.school_yrs,
                       school2 = norm.school_yrs ^ 2,
                       school3 = norm.school_yrs ^ 3,
                       norm.sla_owner = sla_owner - mu.sla_owner,
                       norm.sla_share = sla_share - mu.sla_share,
                       norm.usual_inc = usual_inc - mu.usual_inc,
                       norm.inc_low = inc_low - mu.inc_low,
                       norm.inc_high = inc_high - mu.inc_high)
}

full_imp1 <- normalize(full_imp1)
full_imp2 <- normalize(full_imp2)
full_imp3 <- normalize(full_imp3)
full_imp4 <- normalize(full_imp4)
full_imp5 <- normalize(full_imp5)

rm(list = c("scf.design"))

scf.repdesign <- 
        svrepdesign(
                weights = ~wgt,
                repweights = full_rw[ , -1 ],
                data = imputationList(list(full_imp1, full_imp2, full_imp3, 
                                             full_imp4, full_imp5)),
                scale = 1 ,
                rscales = rep( 1 / 998, 999 ),
                mse = TRUE,
                type = "other",
                combined.weights = TRUE
        )

# Create function to estimate McFadden's Psuedo R-Squared term from each of the
# five regressions, then average those for overall Psuedo R-squared.
export.results <- function(results, filename) {
        y <- scf.MIcombine(results)
        m <- length(results)
        r2 <- NULL
        for (i in 1:m) {
                # Estimate McFadden's Psuedo R-Squared terms
                temp <- 1 - (results[[i]]$deviance/results[[i]]$null.deviance)
                r2 <- c(r2, temp)
        }
        x <- as.data.frame(cbind(coef(y), SE(y)))
        names(x) <- c("Beta", "Sigma")
        x <- mutate(x, Variable = rownames(x),
                    #Apply Halvorsen-Palmquist transformation for dummy variables
                    HP = ifelse(
                            Variable %in% 
                                    c("age", "age2", "age3", "kids", 
                                      "fam_supp_share", "dg_share", "home_share",
                                      "sla_share", "finbus_share", "nmd_share",
                                      "md_share", "fin_score", "usual_inc", 
                                      "norm.school_yrs", "school2", "school3", 
                                      "norm.ismarried", "norm.kids", 
                                      "norm.fam_supp", "norm.fam_supp_share", 
                                      "norm.dg_owner", "norm.dg_share", 
                                      "norm.home_owner", "norm.home_share",
                                      "norm.sla_owner", "norm.sla_share", 
                                      "norm.finbus_owner", "norm.finbus_share",
                                      "norm.nmd_owner", "norm.nmd_share",
                                      "norm.md_owner", "norm.md_share", 
                                      "norm.fin_score", "norm.usual_inc", 
                                      "norm.gift", "norm.healthy", 
                                      "norm.inc_high", "norm.inc_low"), NA, 
                                100 * (exp(Beta * theta) - 1)))
        x[1:length(x[,1]) , 5] = NA
        x[length(x[,1]), 5] = mean(r2)
        names(x)[5] = "Psuedo R2"
        x = x[c(3, 1, 2, 4, 5)]
        write.xlsx(x, filename, sheetName = deparse(substitute(results)), 
                   row.names = FALSE, append = TRUE)
}

exportFile <- paste0(out.dir, "/Weighted Regression Results.xlsx")

#### "Post-Racial" Regressions ####
# Observables explain the racial wealth gap, assumes equal choice sets

pr_88 <- with(scf.repdesign, 
              svyglm(ihs_adj_nw ~ isblack + ishispanic + isother))
export.results(pr_88, exportFile)
rm(pr_88)
gc()

pr_97 <- with(scf.repdesign, 
              svyglm(ihs_adj_nw ~ isblack + ishispanic + isother + age + age2 + 
                             age3 + y1998 + y2001 + y2004 + y2007 + y2010 + 
                             y2013 + b_1900 + b_1905 + b_1910 + b_1915 + b_1920 +
                             b_1925 + b_1930 + b_1935 + b_1945 + b_1950 + 
                             b_1955 + b_1960 + b_1965 + b_1970 + b_1975 + 
                             b_1980 + b_1985 + b_1990))
export.results(pr_97, exportFile)
rm(pr_97)
gc()

pr_98 <- with(scf.repdesign, 
              svyglm(ihs_adj_nw ~ isblack + ishispanic + isother + age + age2 + 
                             age3 + y1998 + y2001 + y2004 + 
                             y2007 + y2010 + y2013 + b_1900 + b_1905 + b_1910 +
                             b_1915 + b_1920 + b_1925 + b_1930 + b_1935 + b_1945 +
                             b_1950 + b_1955 + b_1960 + b_1965 + b_1970 + b_1975 + 
                             b_1980 + b_1985 + b_1990 + no_hs + hs + grad))
export.results(pr_98, exportFile)
rm(pr_98)
gc()

pr_99 <- with(scf.repdesign, 
              svyglm(ihs_adj_nw ~ isblack + ishispanic + isother + age + age2 + 
                             age3 + y1998 + y2001 + y2004 + 
                             y2007 + y2010 + y2013 + b_1900 + b_1905 + b_1910 +
                             b_1915 + b_1920 + b_1925 + b_1930 + b_1935 + b_1945 +
                             b_1950 + b_1955 + b_1960 + b_1965 + b_1970 + b_1975 + 
                             b_1980 + b_1985 + b_1990 + ismarried + kids + 
                             fam_supp + fam_supp_share))
export.results(pr_99, exportFile)
rm(pr_99)
gc()

pr_100 <- with(scf.repdesign, 
              svyglm(ihs_adj_nw ~ isblack + ishispanic + isother + age + age2 + 
                             age3 + y1998 + y2001 + y2004 + 
                             y2007 + y2010 + y2013 + b_1900 + b_1905 + b_1910 +
                             b_1915 + b_1920 + b_1925 + b_1930 + b_1935 + b_1945 +
                             b_1950 + b_1955 + b_1960 + b_1965 + b_1970 + b_1975 + 
                             b_1980 + b_1985 + b_1990 + dg_owner + dg_share + 
                             home_owner + home_share + sla_owner + sla_share + 
                             finbus_owner + finbus_share + nmd_owner + nmd_share + 
                             md_owner + md_share + fin_score))
export.results(pr_100, exportFile)
rm(pr_100)
gc()

pr_102 <- with(scf.repdesign, 
               svyglm(ihs_adj_nw ~ isblack + ishispanic + isother + age + age2 + 
                              age3 + y1998 + y2001 + y2004 + 
                              y2007 + y2010 + y2013 + b_1900 + b_1905 + b_1910 +
                              b_1915 + b_1920 + b_1925 + b_1930 + b_1935 + b_1945 +
                              b_1950 + b_1955 + b_1960 + b_1965 + b_1970 + b_1975 + 
                              b_1980 + b_1985 + b_1990 + inc_high + inc_low + gift + 
                              healthy))
export.results(pr_102, exportFile)
rm(pr_102)
gc()

pr_115 <- with(scf.repdesign, 
               svyglm(ihs_adj_nw ~ isblack + ishispanic + isother + age + age2 + 
                              age3 + y1998 + y2001 + y2004 + 
                              y2007 + y2010 + y2013 + b_1900 + b_1905 + b_1910 +
                              b_1915 + b_1920 + b_1925 + b_1930 + b_1935 + b_1945 +
                              b_1950 + b_1955 + b_1960 + b_1965 + b_1970 + b_1975 + 
                              b_1980 + b_1985 + b_1990 + no_hs + hs + grad + 
                              ismarried + kids + fam_supp + fam_supp_share + 
                              dg_owner + dg_share + home_owner + home_share + 
                              sla_owner + sla_share + finbus_owner + finbus_share +
                              nmd_owner + nmd_share + md_owner + md_share + 
                              fin_score + inc_high + inc_low + gift + healthy))
export.results(pr_115, exportFile)
rm(pr_115)
gc()

pr_116 <- with(scf.repdesign, 
               svyglm(ihs_adj_nw ~ isblack + ishispanic + isother + age + age2 + 
                              age3 +  y1998 + y2001 + y2004 + 
                              y2007 + y2010 + y2013 + b_1900 + b_1905 + b_1910 +
                              b_1915 + b_1920 + b_1925 + b_1930 + b_1935 + b_1945 +
                              b_1950 + b_1955 + b_1960 + b_1965 + b_1970 + b_1975 + 
                              b_1980 + b_1985 + b_1990 + no_hs + hs + grad + 
                              ismarried + kids + fam_supp + fam_supp_share + 
                              dg_owner + dg_share + home_owner + home_share + 
                              sla_owner + sla_share + finbus_owner + finbus_share +
                              nmd_owner + nmd_share + md_owner + md_share + 
                              fin_score + inc_high + inc_low + gift + healthy + nohs.black +
                              nohs.hispanic + nohs.other + hs.black + hs.hispanic +
                              hs.other + grad.black + grad.hispanic + grad.other))
export.results(pr_116, exportFile)
rm(pr_116)
gc()

#### Structural Regressions ####
# Idisyncratic deviations from peer group means explain wealth outcomes.

sr_204 <- with(scf.repdesign, 
              svyglm(ihs_adj_nw ~ isblack + ishispanic + isother))
export.results(sr_204, exportFile)
rm(sr_204)
gc()

sr_213 <- with(scf.repdesign, 
              svyglm(ihs_adj_nw ~ isblack + ishispanic + isother + age + age2 + 
                             age3 + y1998 + y2001 + y2004 + 
                             y2007 + y2010 + y2013 + b_1900 + b_1905 + b_1910 +
                             b_1915 + b_1920 + b_1925 + b_1930 + b_1935 + b_1945 +
                             b_1950 + b_1955 + b_1960 + b_1965 + b_1970 + b_1975 + 
                             b_1980 + b_1985 + b_1990))
export.results(sr_213, exportFile)
rm(sr_213)
gc()

sr_214 <- with(scf.repdesign, 
               svyglm(ihs_adj_nw ~ isblack + ishispanic + isother + age + age2 + 
                              age3 + y1998 + y2001 + y2004 + 
                              y2007 + y2010 + y2013 + b_1900 + b_1905 + b_1910 +
                              b_1915 + b_1920 + b_1925 + b_1930 + b_1935 + b_1945 +
                              b_1950 + b_1955 + b_1960 + b_1965 + b_1970 + b_1975 + 
                              b_1980 + b_1985 + b_1990 + norm.school_yrs +
                              school2 + school3))
export.results(sr_214, exportFile)
rm(sr_214)
gc()

sr_215 <- with(scf.repdesign, 
               svyglm(ihs_adj_nw ~ isblack + ishispanic + isother + age + age2 + 
                              age3 +  y1998 + y2001 + y2004 + 
                              y2007 + y2010 + y2013 + b_1900 + b_1905 + b_1910 +
                              b_1915 + b_1920 + b_1925 + b_1930 + b_1935 + b_1945 +
                              b_1950 + b_1955 + b_1960 + b_1965 + b_1970 + b_1975 + 
                              b_1980 + b_1985 + b_1990 + norm.ismarried +
                              norm.kids + norm.fam_supp + norm.fam_supp_share))
export.results(sr_215, exportFile)
rm(sr_215)
gc()

sr_216 <- with(scf.repdesign, 
               svyglm(ihs_adj_nw ~ isblack + ishispanic + isother + age + age2 + 
                              age3 + y1998 + y2001 + y2004 + 
                              y2007 + y2010 + y2013 + b_1900 + b_1905 + b_1910 +
                              b_1915 + b_1920 + b_1925 + b_1930 + b_1935 + b_1945 +
                              b_1950 + b_1955 + b_1960 + b_1965 + b_1970 + b_1975 + 
                              b_1980 + b_1985 + b_1990 + norm.dg_owner + 
                              norm.dg_share + norm.home_owner + norm.home_share + 
                              norm.sla_owner + norm.sla_share + norm.finbus_owner + 
                              norm.finbus_share + norm.nmd_owner + norm.nmd_share + 
                              norm.md_owner + norm.md_share + norm.fin_score))
export.results(sr_216, exportFile)
rm(sr_216)
gc()

sr_218 <- with(scf.repdesign, 
               svyglm(ihs_adj_nw ~ isblack + ishispanic + isother + age + age2 + 
                              age3 +  y1998 + y2001 + y2004 + 
                              y2007 + y2010 + y2013 + b_1900 + b_1905 + b_1910 +
                              b_1915 + b_1920 + b_1925 + b_1930 + b_1935 + b_1945 +
                              b_1950 + b_1955 + b_1960 + b_1965 + b_1970 + b_1975 + 
                              b_1980 + b_1985 + b_1990 + norm.inc_high + norm.inc_low +
                              norm.gift + norm.healthy))
export.results(sr_218, exportFile)
rm(sr_218)
gc()

sr_231 <- with(scf.repdesign, 
               svyglm(ihs_adj_nw ~ isblack + ishispanic + isother + age + age2 + 
                              age3 + y1998 + y2001 + y2004 + 
                              y2007 + y2010 + y2013 + b_1900 + b_1905 + b_1910 +
                              b_1915 + b_1920 + b_1925 + b_1930 + b_1935 + b_1945 +
                              b_1950 + b_1955 + b_1960 + b_1965 + b_1970 + b_1975 + 
                              b_1980 + b_1985 + b_1990 + norm.school_yrs +
                              school2 + school3 + norm.ismarried + norm.kids + 
                              norm.fam_supp + norm.fam_supp_share + norm.dg_owner + 
                              norm.dg_share + norm.home_owner + norm.home_share + 
                              norm.sla_owner + norm.sla_share + norm.finbus_owner + 
                              norm.finbus_share + norm.nmd_owner + norm.nmd_share + 
                              norm.md_owner + norm.md_share + norm.fin_score + 
                              norm.inc_high + norm.inc_low + norm.gift + norm.healthy))
export.results(sr_231, exportFile)
rm(sr_231)
gc()

sr_232 <- with(scf.repdesign, 
               svyglm(ihs_adj_nw ~ isblack + ishispanic + isother + age + age2 + 
                              age3 +  y1998 + y2001 + y2004 + 
                              y2007 + y2010 + y2013 + b_1900 + b_1905 + b_1910 +
                              b_1915 + b_1920 + b_1925 + b_1930 + b_1935 + b_1945 +
                              b_1950 + b_1955 + b_1960 + b_1965 + b_1970 + b_1975 + 
                              b_1980 + b_1985 + b_1990 + norm.school_yrs +
                              school2 + school3 + norm.ismarried + norm.kids + 
                              norm.fam_supp + norm.fam_supp_share + norm.dg_owner + 
                              norm.dg_share + norm.home_owner + norm.home_share + 
                              norm.sla_owner + norm.sla_share + norm.finbus_owner + 
                              norm.finbus_share + norm.nmd_owner + norm.nmd_share + 
                              norm.md_owner + norm.md_share + norm.fin_score + 
                              norm.inc_high + norm.inc_low + norm.gift + norm.healthy + 
                              nohs.black + nohs.hispanic + nohs.other + hs.black +
                              hs.hispanic + hs.other + grad.black + grad.hispanic + 
                              grad.other))
export.results(sr_232, exportFile)
rm(sr_232)
gc()

